/*******************************************************************
* - Project          : 2024 summner short course
* - File name        : TbTop_Lab3_1_wRdChk.v
* - Description      : Testbench top for Lab1
* - Owner            : Hyuntaek.Jung
* - Revision history : 1) 2024.07.29 : Initial release
********************************************************************/

`timescale 1ns/10ps

module TbTop_Lab3_1_wRdChk;


  /***********************************************
  // wire & register
  ***********************************************/
  reg              iClk;
  reg              iRsn;


  reg              iPsel;
  reg              iPenable;
  reg              iPwrite;
  reg  [15:0]      iPaddr;

  reg  [31:0]      iPwdata;
  wire [31:0]      oPrdata;



  /***********************************************
  // TrdPrjTop.v  instantiation
  ***********************************************/
  Lab3_1_Top A_Lab3_1_Top (
  // Clock & reset
  .iClk            (iClk),
  .iRsn            (iRsn),


  // APB interface
  .iPsel           (iPsel),
  .iPenable        (iPenable),
  .iPwrite         (iPwrite),
  .iPaddr          (iPaddr[15:0]),

  .iPwdata         (iPwdata[31:0]),
  .oPrdata         (oPrdata[31:0])
  );



  /***********************************************
  // Clock define
  ***********************************************/
  initial
  begin
    iClk = 1'b0;
  end


  always
  begin
    // 100MHz clock
    #5 iClk <= ~iClk;
  end



  /***********************************************
  // Sync. & active low reset define
  ***********************************************/
  initial
  begin
    iRsn = 1'b1;

    repeat (  5) @(posedge iClk);
    iRsn = 1'b0;

    repeat (  2) @(posedge iClk); 
    $display("**** Reset released !!! ****");
    iRsn = 1'b1;
  end



  /***********************************************
  // APB write task
  ***********************************************/
  task apb_write (
    input  [15:0]  addr,     // Write address
    input  [31:0]  data      // Read data
  );
  begin

    iPsel    = 1'b1;
    iPenable = 1'b0;
    iPwrite  = 1'b1;
    iPaddr   = addr[15:0];
    iPwdata  = data[31:0];

    @(posedge iClk);
    iPenable = 1'b1;

    @(posedge iClk);
    $display("**** Write 0x%h at addr 0x%h !!! ****", data[31:0], addr[15:0]);
    iPsel    = 1'b0;
    iPenable = 1'b0;

  end
  endtask


    
  /***********************************************
  // APB read task
  ***********************************************/
  task apb_read (
    input  [15:0]  addr,     // Read address
    input  [31:0]  data      // Expected data
  );
  begin

    iPsel    = 1'b1;
    iPenable = 1'b0;
    iPwrite  = 1'b0;
    iPaddr   = addr[15:0];

    @(posedge iClk);
    iPenable = 1'b1;

/**
    @(posedge iClk);
    $display("**** Read  0x%h from addr 0x%h !!! ****", oPrdata[31:0], addr[15:0]);
**/
    @(posedge iClk);
    $display("**** Read  0x%h & expected 0x%h from addr 0x%h !!! ****", oPrdata[31:0], data[31:0], addr[15:0]);

    if (oPrdata[31:0] == data[31:0])
    begin
      $display ("     OOOO Read data Passed !!! OOOO");
    end
    else
    begin
      $display ("     XXXX Read data Failed !!! XXXX");
      $display ("     ---> Must debug this  !!! <---");
    end
      
    iPsel    = 1'b0;
    iPenable = 1'b0;

  end
  endtask




  /***********************************************
  // Interrupt make
  ***********************************************/
  initial
  begin

    iPsel    =  1'h0;
    iPenable =  1'h0;
    iPwrite  =  1'h0;
    iPaddr   = 16'h0;
    iPwdata  = 32'h0;



    // Memory data write @0x4000
    repeat (100) @(posedge iClk);
    apb_write(16'h4000, 32'h11111111); 

    // Memory data write @0x4004
    repeat (100) @(posedge iClk);
    apb_write(16'h4004, 32'h22222222); 

    // Memory data write @0x4008
    repeat (100) @(posedge iClk);
    apb_write(16'h4008, 32'h33333333); 
  //apb_write(16'h4008, 32'h30303030); 

    // Memory data write @0x400C
    repeat (100) @(posedge iClk);
    apb_write(16'h400C, 32'h44444444); 



    // Memory data read @0x4000
    repeat (100) @(posedge iClk);
    apb_read(16'h4000, 32'h11111111); 

    // Memory data read @0x4004
    repeat (100) @(posedge iClk);
    apb_read(16'h4004, 32'h22222222); 

    // Memory data read @0x4008
    repeat (100) @(posedge iClk);
    apb_read(16'h4008, 32'h33333333); 

    // Memory data read @0x400C
    repeat (100) @(posedge iClk);
    apb_read(16'h400C, 32'h44444444); 
  //apb_read(16'h400C, 32'h40404040); 



    // Finish
    repeat (100) @(posedge iClk);
    $finish;

  end



  /***********************************************
  // VCD dump
  ***********************************************/
  initial
  begin
    $shm_open("/home/Summer24426/ShortCourse_Hyuntaek/Lab3_1/TestBench/Dump/Lab3_1_wRdChk.shm");
    $shm_probe("AC");
  end


endmodule
